/************ Drop: Tables ***************/
DROP TABLE argument, command, command_parameter, job_management, 
command_queue_list, command_result, command_status_type_description,
environment, input_file, job, job_child, job_command, job_status,
job_status_type_description, output_file,
output_sandbox_dest_URI, job_lease CASCADE;

/************ Drop: Sequences ***************/
DROP SEQUENCE 
argument_id_seq,
command_id_seq,
command_parameter_id_seq,
inputfile_id_seq,
job_command_id_seq,
jobcommand_id_seq,
job_management_id_seq,
job_status_id_seq,
jobstatus_id_seq,
outputfile_id_seq,
outputsandboxdesturi_id_seq,
status_id_seq;

/************ Update: Tables ***************/

/******************** Add Table: argument ************************/

/* Build Table Structure */
CREATE TABLE argument
(
    value VARCHAR(256) NOT NULL,
    jobId VARCHAR(14) NOT NULL
);

/* Table Items: argument */

/* Set Comments */
COMMENT ON TABLE argument IS 'Table for arguments of a job';

/******************** Add Table: command ************************/

/* Build Table Structure */
CREATE TABLE command
(
    id BIGINT NOT NULL,
    name VARCHAR(256) NOT NULL,
    category VARCHAR(256) NULL,
    description TEXT NULL,
    statusType INTEGER NOT NULL,
    failureReason TEXT NULL,
    cmdExecutorName VARCHAR(256) NULL,
    userId VARCHAR(256) NOT NULL,
    startSchedulingTime TIMESTAMP NULL,
    startProcessingTime TIMESTAMP NULL,
    executionCompletedTime TIMESTAMP NULL,
    creationTime TIMESTAMP NOT NULL DEFAULT now()
);

/* Table Items: command */
ALTER TABLE command ADD CONSTRAINT pkcommand
    PRIMARY KEY (id);

/* Set Comments */
COMMENT ON TABLE command IS 'Table for commands of a job';

/******************** Add Table: command_parameter ************************/

/* Build Table Structure */
CREATE TABLE command_parameter
(
    name VARCHAR(256) NOT NULL,
    value VARCHAR(256) NOT NULL,
    commandId INTEGER NOT NULL,
    id BIGINT NOT NULL
);

/* Table Items: command_parameter */
ALTER TABLE command_parameter ADD CONSTRAINT pkcommand_parameter
    PRIMARY KEY (id);

/* Set Comments */
COMMENT ON TABLE command_parameter IS 'Table for comman parameters';

/******************** Add Table: command_queue_list ************************/

/* Build Table Structure */
CREATE TABLE command_queue_list
(
    name VARCHAR(256) NOT NULL
);

/******************** Add Table: command_result ************************/

/* Build Table Structure */
CREATE TABLE command_result
(
    id INTEGER NOT NULL,
    name VARCHAR(256) NOT NULL,
    value VARCHAR(256) NOT NULL,
    commandId INTEGER NOT NULL
);

/* Table Items: command_result */
ALTER TABLE command_result ADD CONSTRAINT pkcommand_result
    PRIMARY KEY (id);

/* Set Comments */
COMMENT ON TABLE command_result IS 'Table for command results';

/******************** Add Table: command_status_type_description ************************/

/* Build Table Structure */
CREATE TABLE command_status_type_description
(
    type INTEGER NOT NULL,
    name VARCHAR(256) NOT NULL
);

/* Table Items: command_status_type_description */
ALTER TABLE command_status_type_description ADD CONSTRAINT pkcommand_status_type_description
    PRIMARY KEY (type);

/* Set Comments */
COMMENT ON TABLE command_status_type_description IS 'Table for coding allowed states for a command';

/******************** Add Table: environment ************************/

/* Build Table Structure */
CREATE TABLE environment
(
    name VARCHAR(256) NOT NULL,
    value VARCHAR(256) NOT NULL,
    jobId VARCHAR(14) NOT NULL
);

/* Table Items: environment */

/* Set Comments */
COMMENT ON TABLE environment IS 'Table for env variables';

/******************** Add Table: input_file ************************/

/* Build Table Structure */
CREATE TABLE input_file
(
    value VARCHAR(256) NOT NULL,
    jobId VARCHAR(14) NOT NULL
);

/******************** Add Table: job ************************/

/* Build Table Structure */
CREATE TABLE job
(
    creamURL VARCHAR(256) NOT NULL,
    id VARCHAR(14) NOT NULL,
    cerequirements VARCHAR(256) NULL,
    virtualOrganization VARCHAR(256) NULL,
    userId VARCHAR(256) NOT NULL,
    batchSystem VARCHAR(256) NOT NULL,
    queue VARCHAR(256) NOT NULL,
    standardInput VARCHAR(256) NULL,
    standardOutput VARCHAR(256) NULL,
    standardError VARCHAR(256) NULL,
    executable VARCHAR(256) NOT NULL,
    delegationProxyCertPath VARCHAR(256) NULL,
    authNProxyCertPath VARCHAR(256) NULL,
    hlrLocation VARCHAR(256) NULL,
    loggerDestURI VARCHAR(256) NULL,
    tokenURL VARCHAR(256) NULL,
    perusalFilesDestURI VARCHAR(256) NULL,
    perusalListFileURI VARCHAR(256) NULL,
    perusalTimeInterval INTEGER NULL,
    nodes INTEGER NULL,
    prologueArguments VARCHAR(256) NULL,
    prologue VARCHAR(256) NULL,
    epilogue VARCHAR(256) NULL,
    epilogueArguments VARCHAR(256) NULL,
    sequenceCode VARCHAR(256) NULL,
    lrmsJobId VARCHAR(256) NULL,
    lrmsAbsLayerJobId VARCHAR(256) NULL,
    gridJobId VARCHAR(256) NULL,
    iceId VARCHAR(256) NULL,
    fatherJobId VARCHAR(14) NULL,
    ceId VARCHAR(256) NULL,
    type VARCHAR(256) NOT NULL,
    creamInputSandboxURI VARCHAR(256) NULL,
    creamOutputSandboxURI VARCHAR(256) NULL,
    sandboxBasePath VARCHAR(256) NULL,
    inputSandboxBaseURI VARCHAR(256) NULL,
    outputSandboxBaseDestURI VARCHAR(256) NULL,
    workerNode VARCHAR(256) NULL,
    jdl TEXT NOT NULL,
    localUser VARCHAR(256) NOT NULL,
    delegationProxyId VARCHAR(256) NULL,
    delegationProxyInfo TEXT NULL,
    workingDirectory VARCHAR(256) NOT NULL,
    leaseId VARCHAR(256) NULL
);

/* Table Items: job */
ALTER TABLE job ADD CONSTRAINT pkjob
    PRIMARY KEY (id);

/* Set Comments */
COMMENT ON TABLE job IS 'Table for representing static and dynamic info of a job';

/* Add Indexes for: job */
CREATE INDEX job_proxyDelegationId_Idx ON job (delegationProxyId);
CREATE INDEX job_userId_Idx ON job (userId);

/******************** Add Table: job_child ************************/

/* Build Table Structure */
CREATE TABLE job_child
(
    id VARCHAR(14) NOT NULL,
    fatherJobId VARCHAR(14) NOT NULL
);

/******************** Add Table: job_command ************************/

/* Build Table Structure */
CREATE TABLE job_command
(
    id BIGINT NOT NULL,
    description TEXT NULL,
    statusType INTEGER NOT NULL,
    failureReason TEXT NULL,
    jobId VARCHAR(14) NOT NULL,
    cmdExecutorName VARCHAR(256) NULL,
    userId VARCHAR(256) NOT NULL,
    startSchedulingTime TIMESTAMP NULL,
    startProcessingTime TIMESTAMP NULL,
    executionCompletedTime TIMESTAMP NULL,
    creationTime TIMESTAMP NOT NULL,
    type INTEGER NOT NULL
);

/* Table Items: job_command */
ALTER TABLE job_command ADD CONSTRAINT pkjob_command
    PRIMARY KEY (id);

/******************** Add Table: job_lease ************************/

/* Build Table Structure */
CREATE TABLE job_lease
(
    leaseId VARCHAR(256) NOT NULL,
    userId VARCHAR(256) NOT NULL,
    leaseTime TIMESTAMP NOT NULL
);

/* Table Items: job_lease */
ALTER TABLE job_lease ADD CONSTRAINT pkjob_lease
    PRIMARY KEY (leaseId, userId);

/* Set Comments */
COMMENT ON TABLE job_lease IS 'Job lease table';

/******************** Add Table: job_management ************************/

/* Build Table Structure */
CREATE TABLE job_management
(
    id BIGINT NOT NULL,
    commandId BIGINT NOT NULL,
    isScheduled BOOL NOT NULL
);

/* Table Items: job_management */
ALTER TABLE job_management ADD CONSTRAINT pkjob_management
    PRIMARY KEY (id);

/* Set Comments */
COMMENT ON TABLE job_management IS 'Table for command queue';

/******************** Add Table: job_status ************************/

/* Build Table Structure */
CREATE TABLE job_status
(
    type INTEGER NOT NULL,
    exitCode VARCHAR(256) NULL,
    failureReason TEXT NULL,
    description TEXT NULL,
    "timestamp" TIMESTAMP NOT NULL,
    jobId VARCHAR(14) NOT NULL,
    id BIGINT NOT NULL
);

/* Table Items: job_status */
ALTER TABLE job_status ADD CONSTRAINT pkjob_status
    PRIMARY KEY (id);

/* Set Comments */
COMMENT ON TABLE job_status IS 'Table for status changes associated to a job';

/******************** Add Table: job_status_type_description ************************/

/* Build Table Structure */
CREATE TABLE job_status_type_description
(
    type INTEGER NOT NULL,
    name VARCHAR(256) NOT NULL
);

/* Table Items: job_status_type_description */
ALTER TABLE job_status_type_description ADD CONSTRAINT pkjob_status_type_description
    PRIMARY KEY (type);

/* Set Comments */
COMMENT ON TABLE job_status_type_description IS 'Table for coding allowed states for a job';

/******************** Add Table: output_file ************************/

/* Build Table Structure */
CREATE TABLE output_file
(
    value VARCHAR(256) NOT NULL,
    jobId VARCHAR(14) NOT NULL
);

/******************** Add Table: output_sandbox_dest_URI ************************/

/* Build Table Structure */
CREATE TABLE output_sandbox_dest_URI
(
    value VARCHAR(256) NOT NULL,
    jobId VARCHAR(14) NOT NULL
);


/************ Update: Sequences ***************/
CREATE SEQUENCE argument_id_seq INCREMENT 1;
CREATE SEQUENCE command_id_seq INCREMENT 1;
CREATE SEQUENCE command_parameter_id_seq INCREMENT 1;
CREATE SEQUENCE inputfile_id_seq INCREMENT 1;
CREATE SEQUENCE job_command_id_seq INCREMENT 1;
CREATE SEQUENCE job_management_id_seq INCREMENT 1;
CREATE SEQUENCE job_status_id_seq INCREMENT 1;
CREATE SEQUENCE jobcommand_id_seq INCREMENT 1;
CREATE SEQUENCE jobstatus_id_seq INCREMENT 1;
CREATE SEQUENCE outputfile_id_seq INCREMENT 1;
CREATE SEQUENCE outputsandboxdesturi_id_seq INCREMENT 1;
CREATE SEQUENCE status_id_seq INCREMENT 1;


/************ Add Foreign Keys to Database ***************/

/************ Foreign Key: fk_argument_jobId_job_id ***************/
ALTER TABLE argument ADD CONSTRAINT fk_argument_jobId_job_id
    FOREIGN KEY (jobId) REFERENCES job (id) ON UPDATE NO ACTION ON DELETE NO ACTION;

/************ Foreign Key: fk_jobCommand_type_cStatus_type ***************/
ALTER TABLE command ADD CONSTRAINT fk_jobCommand_type_cStatus_type
    FOREIGN KEY (statusType) REFERENCES command_status_type_description (type) ON UPDATE NO ACTION ON DELETE NO ACTION;

/************ Foreign Key: fk_commandParameter_commandId_jobCommand_id ***************/
ALTER TABLE command_parameter ADD CONSTRAINT fk_commandParameter_commandId_jobCommand_id
    FOREIGN KEY (commandId) REFERENCES command (id) ON UPDATE NO ACTION ON DELETE NO ACTION;

/************ Foreign Key: fk_commandResult_commandId_jobCommand_id ***************/
ALTER TABLE command_result ADD CONSTRAINT fk_commandResult_commandId_jobCommand_id
    FOREIGN KEY (commandId) REFERENCES command (id) ON UPDATE NO ACTION ON DELETE NO ACTION;

/************ Foreign Key: fk_environment_jobId_job_id ***************/
ALTER TABLE environment ADD CONSTRAINT fk_environment_jobId_job_id
    FOREIGN KEY (jobId) REFERENCES job (id) ON UPDATE NO ACTION ON DELETE NO ACTION;

/************ Foreign Key: fk_inputFile_jobId_job_id ***************/
ALTER TABLE input_file ADD CONSTRAINT fk_inputFile_jobId_job_id
    FOREIGN KEY (jobId) REFERENCES job (id) ON UPDATE NO ACTION ON DELETE NO ACTION;

/************ Foreign Key: fk_childJob_fatherJobId_job_id ***************/
ALTER TABLE job_child ADD CONSTRAINT fk_childJob_fatherJobId_job_id
    FOREIGN KEY (fatherJobId) REFERENCES job (id) ON UPDATE NO ACTION ON DELETE NO ACTION;

/************ Foreign Key: fk_job_command_jobId_job_id ***************/
ALTER TABLE job_command ADD CONSTRAINT fk_job_command_jobId_job_id
    FOREIGN KEY (jobId) REFERENCES job (id) ON UPDATE NO ACTION ON DELETE NO ACTION;

/************ Foreign Key: fk_command_queue_jobId_command_id ***************/
ALTER TABLE job_management ADD CONSTRAINT fk_command_queue_jobId_command_id
    FOREIGN KEY (commandId) REFERENCES command (id) ON UPDATE NO ACTION ON DELETE NO ACTION;

/************ Foreign Key: fk_jobStatus_jobId_job_id ***************/
ALTER TABLE job_status ADD CONSTRAINT fk_jobStatus_jobId_job_id
    FOREIGN KEY (jobId) REFERENCES job (id) ON UPDATE NO ACTION ON DELETE NO ACTION;

/************ Foreign Key: fk_jobStatus_type_jStatus_type ***************/
ALTER TABLE job_status ADD CONSTRAINT fk_jobStatus_type_jStatus_type
    FOREIGN KEY (type) REFERENCES job_status_type_description (type) ON UPDATE NO ACTION ON DELETE NO ACTION;

/************ Foreign Key: fk_outputFile_jobId_job_id ***************/
ALTER TABLE output_file ADD CONSTRAINT fk_outputFile_jobId_job_id
    FOREIGN KEY (jobId) REFERENCES job (id) ON UPDATE NO ACTION ON DELETE NO ACTION;

/************ Foreign Key: fk_outputSandboxDestURI_jobId_job_id ***************/
ALTER TABLE output_sandbox_dest_URI ADD CONSTRAINT fk_outputSandboxDestURI_jobId_job_id
    FOREIGN KEY (jobId) REFERENCES job (id) ON UPDATE NO ACTION ON DELETE NO ACTION;
    
/************ Insert values in command_status_type_description ***************/    
insert into command_status_type_description (type, name) values (0, 'CREATED');
insert into command_status_type_description (type, name) values (1, 'QUEUED');
insert into command_status_type_description (type, name) values (2, 'SCHEDULED');
insert into command_status_type_description (type, name) values (3, 'RESCHEDULED');
insert into command_status_type_description (type, name) values (4, 'PROCESSING');
insert into command_status_type_description (type, name) values (5, 'REMOVED');
insert into command_status_type_description (type, name) values (6, 'SUCCESSFULL');
insert into command_status_type_description (type, name) values (7, 'ERROR');

/************ Insert values in status_type_description ***************/    
insert into job_status_type_description (type, name) values (0, 'REGISTERED');
insert into job_status_type_description (type, name) values (1, 'PENDING');
insert into job_status_type_description (type, name) values (2, 'IDLE');
insert into job_status_type_description (type, name) values (3, 'RUNNING');
insert into job_status_type_description (type, name) values (4, 'REALLY_RUNNING');
insert into job_status_type_description (type, name) values (5, 'CANCELLED');
insert into job_status_type_description (type, name) values (6, 'HELD');
insert into job_status_type_description (type, name) values (7, 'DONE_OK');
insert into job_status_type_description (type, name) values (8, 'DONE_FAILED');
insert into job_status_type_description (type, name) values (9, 'PURGED');
insert into job_status_type_description (type, name) values (10, 'ABORTED');    